/*  Skeleton Program code for the AQA A Level Paper 1 Summer 2020 examination
    this code should be used in conjunction with the Preliminary Material
    written by the AQA Programmer Team
    developed in the NetBeans IDE 8.1 environment
*/
package aqa.food.magnate;

import java.util.ArrayList;
import java.util.List;
import java.util.Random;
 
class Household {
    protected float chanceEatOutPerDay;
    protected int xCoord, yCoord, id;
    protected static int nextid = 1;
    private static Random rnd = new Random();

    public Household(int x, int y) {
        xCoord = x;
        yCoord = y;
        chanceEatOutPerDay = rnd.nextFloat();
        id = nextid;
        nextid += 1;
    }

    public String getDetails() {
        String details;
        details = id + "     Coordinates: (" + xCoord + ", " + yCoord + ")     Eat out probability: " + chanceEatOutPerDay;
        return details;
    }

    public float getChanceEatOut() {
        return chanceEatOutPerDay;
    }

    public int getX() {
        return xCoord;
    }

    public int getY() {
        return yCoord;
    }
}

class Settlement {
    protected int startNoOfHouseholds, xSize, ySize;
    protected List<Household> households = new ArrayList();
    private static Random rnd = new Random();

    public Settlement() {
        xSize = 1000;
        ySize = 1000;
        startNoOfHouseholds = 250;
        createHouseholds();
    }

    public int getNumberOfHouseholds() {
        return households.size();
    }

    public int getXSize() {
        return xSize;
    }

    public int getYSize() {
        return ySize;
    }

    public int [] getRandomLocation() {
        int[] location = new int[2];
        location[0] = (int)(rnd.nextFloat() * xSize);
        location[1] = (int)(rnd.nextFloat() * ySize);
        return location;
    }

    protected void createHouseholds() {
        for (int count = 0; count < startNoOfHouseholds; count++) {
            addHousehold();
        }
    }

    public void addHousehold() {
        int x, y;
        int[] tempLocation = getRandomLocation();
        x = tempLocation[0];
        y = tempLocation[1];
        Household temp = new Household(x, y);
        households.add(temp);
    }

    public void displayHouseholds() {
        Console.writeLine(System.lineSeparator() + "**********************************");
        Console.writeLine("*** Details of all households: ***");
        Console.writeLine("**********************************" + System.lineSeparator());
        for(Household h : households) {
            Console.writeLine(h.getDetails());
        }
        Console.writeLine();
    }

    public boolean findOutIfHouseholdEatsOut(int householdNo, int x, int y) {
        float eatOutRNo = rnd.nextFloat();
        if (eatOutRNo < households.get(householdNo).getChanceEatOut()) {
            return true;
        } else {
            return false;
        }
    }

    public int[] getHouseholdLocation(int householdNo) {
        int[] location = new int[2];
        location[0] = households.get(householdNo).getX();
        location[1] = households.get(householdNo).getY();
        return location;
    }
}

class LargeSettlement extends Settlement {
    public LargeSettlement(int extraxSize, int extraySize, int extraHouseholds) {
        super();
        super.xSize += extraxSize;
        super.ySize += extraySize;
        super.startNoOfHouseholds += extraHouseholds;
        for (int count = 1; count <= extraHouseholds; count++) {
            addHousehold();
        }
    }
}

class Outlet {
    protected int visitsToday, xCoord, yCoord, capacity, maxCapacity;
    protected float dailyCosts;
    private static Random rnd = new Random();

    public Outlet(int xCoord, int yCoord, int maxCapacityBase) {
        this.xCoord = xCoord;
        this.yCoord = yCoord;
        capacity = (int)(maxCapacityBase * 0.6f);
        maxCapacity = maxCapacityBase + (int)(rnd.nextFloat() * 50) - (int)(rnd.nextFloat() * 50);
        dailyCosts = maxCapacityBase * 0.2f + capacity * 0.5f + 100;
        newDay();
    }

    public int GetCapacity() {
        return capacity;
    }

    public int getX() {
        return xCoord;
    }

    public int getY() {
        return yCoord;
    }

    public void alterDailyCost(float amount) {
        dailyCosts += amount;
    }

    public int alterCapacity(int change) {
        int oldCapacity = capacity;
        capacity += change;
        if (capacity > maxCapacity) {
            capacity = maxCapacity;
            return maxCapacity - oldCapacity;
        } else if (capacity < 0) {
            capacity = 0;
        }
        dailyCosts = maxCapacity * 0.2f + capacity * 0.5f + 100;
        return change;
    }

    public void incrementVisits() {
        visitsToday += 1;
    }

    public void newDay() {
        visitsToday = 0;
    }

    public float calculateDailyProfitLoss(float avgCostPerMeal, float avgPricePerMeal) {
        return (avgPricePerMeal - avgCostPerMeal) * visitsToday - dailyCosts;
    }

    public String getDetails() {
        String details;
        details = "Coordinates: (" + xCoord + ", " + yCoord + ")     Capacity: " + capacity + "      Maximum Capacity: ";
        details += maxCapacity + "      Daily Costs: " + dailyCosts + "      Visits today: " + visitsToday;
        return details;
    }
}

class Company {
    protected String name, category;
    protected float balance, reputationScore, avgCostPerMeal, avgPricePerMeal, dailyCosts, familyOutletCost, fastFoodOutletCost, namedChefOutletCost, fuelCostPerUnit, baseCostOfDelivery;
    protected List<Outlet> outlets = new ArrayList();
    protected int familyFoodOutletCapacity, fastFoodOutletCapacity, namedChefOutletCapacity;
    private static Random rnd = new Random();

    public Company(String name, String category, float balance, int x, int y, float fuelCostPerUnit, float baseCostOfDelivery) {
        familyOutletCost = 1000;
        fastFoodOutletCost = 2000;
        namedChefOutletCost = 15000;
        familyFoodOutletCapacity = 150;
        fastFoodOutletCapacity = 200;        
        namedChefOutletCapacity = 50;
        this.name = name;
        this.category = category;
        this.balance = balance;
        this.fuelCostPerUnit = fuelCostPerUnit;
        this.baseCostOfDelivery = baseCostOfDelivery;
        reputationScore = 100;
        dailyCosts = 100;
        if (this.category.equals("fast food")) {
            avgCostPerMeal = 5;
            avgPricePerMeal = 10;
            reputationScore += rnd.nextFloat() * 10 - 8;
        } else if (this.category.equals("family")) {
            avgCostPerMeal = 12;
            avgPricePerMeal = 14;
            reputationScore += rnd.nextFloat() * 30 - 5;
        } else {
            avgCostPerMeal = 20;
            avgPricePerMeal = 40;
            reputationScore += rnd.nextFloat() * 50;
        }
        openOutlet(x, y);
    }

    public String getName() {
        return name;
    }

    public int getNumberOfOutlets() {
        return outlets.size();
    }

    public float getReputationScore() {
        return reputationScore;
    }

    public void alterDailyCosts(float change) {
        dailyCosts += change;
    }

    public void alterAvgCostPerMeal(float change) {
        avgCostPerMeal += change;
    }

    public void alterFuelCostPerUnit(float change) {
        fuelCostPerUnit += change;
    }

    public void alterReputation(float change) {
        reputationScore += change;
    }

    public void newDay() {
        for(Outlet o : outlets) {
            o.newDay();
        }
    }

    public void addVisitToNearestOutlet(int x, int y) {
        int nearestOutlet = 0;
        float nearestOutletDistance, currentDistance;
        nearestOutletDistance = (float)Math.sqrt(Math.pow(outlets.get(0).getX() - x, 2) + Math.pow(outlets.get(0).getY() - y, 2));
        for (int current = 1; current < outlets.size(); current++) {
            currentDistance = (float)Math.sqrt(Math.pow(outlets.get(current).getX() - x, 2) + Math.pow(outlets.get(current).getY() - y, 2));
            if (currentDistance < nearestOutletDistance) {
                nearestOutletDistance = currentDistance;
                nearestOutlet = current;
            }
        }
        outlets.get(nearestOutlet).incrementVisits();
    }

    public String getDetails() {
        String details = "";
        details += "Name: " + name + System.lineSeparator() + "Type of business: " + category + System.lineSeparator();
        details += "Current balance: " + balance + System.lineSeparator() + "Average cost per meal: " + avgCostPerMeal + System.lineSeparator();
        details += "Average price per meal: " + avgPricePerMeal + System.lineSeparator() + "Daily costs: " + dailyCosts + System.lineSeparator();
        details += "Delivery costs: " + calculateDeliveryCost() + System.lineSeparator() + "Reputation: " + reputationScore + System.lineSeparator() + System.lineSeparator();
        details += "Number of outlets: " + outlets.size() + System.lineSeparator() + "Outlets" + System.lineSeparator();
        for (int current = 1; current <= outlets.size(); current++) {
            details += current + ". " + outlets.get(current - 1).getDetails() + System.lineSeparator();
        }
        return details;
    }

    public String processDayEnd() {
        String details = "";
        float profitLossFromOutlets = 0;
        float profitLossFromThisOutlet = 0;
        float deliveryCosts;
        if (outlets.size() > 1) {
            deliveryCosts = baseCostOfDelivery + calculateDeliveryCost();
        } else {
            deliveryCosts = baseCostOfDelivery;
        }
        details += "Daily costs for company: " + dailyCosts + System.lineSeparator() + "Cost for delivering produce to outlets: " + deliveryCosts + System.lineSeparator();
        for (int current = 0; current < outlets.size(); current++) {
            profitLossFromThisOutlet = outlets.get(current).calculateDailyProfitLoss(avgCostPerMeal, avgPricePerMeal);
            details += "Outlet " + (current + 1) + " profit/loss: " + profitLossFromThisOutlet + System.lineSeparator();
            profitLossFromOutlets += profitLossFromThisOutlet;
        }
        details += "Previous balance for company: " + balance + System.lineSeparator();
        balance += profitLossFromOutlets - dailyCosts - deliveryCosts;
        details += "New balance for company: " + balance;
        return details;
    }

    public boolean closeOutlet(int id) {
        boolean closeCompany  = false;
        outlets.remove(id);
        if (outlets.isEmpty()) {
            closeCompany = true;
        }
        return closeCompany;
    }

    public void expandOutlet(int id) {
        int change, result;
        Console.write("Enter amount you would like to expand the capacity by: ");
        change = Integer.parseInt(Console.readLine());
        result = outlets.get(id).alterCapacity(change);
        if (result == change) {
            Console.writeLine("Capacity adjusted.");
        } else {
            Console.writeLine("Only some of that capacity added, outlet now at maximum capacity.");
        }
    }

    public void openOutlet(int x, int y) {
        int capacity;
        if (category == "fast food") {
            balance -= fastFoodOutletCost;
            capacity = fastFoodOutletCapacity;
        } else if (category == "family") {
            balance -= familyOutletCost;
            capacity = familyFoodOutletCapacity;
        } else {
            balance -= namedChefOutletCost;
            capacity = namedChefOutletCapacity;
        }
        Outlet newOutlet = new Outlet(x, y, capacity);
        outlets.add(newOutlet);
    }

    private List<Integer> getListOfOutlets() {
        List<Integer> temp = new ArrayList();
        for (int current = 0; current < outlets.size(); current++) {
            temp.add(current);
        }
        return temp;
    }

    private float getDistanceBetweenTwoOutlets(int outlet1, int outlet2) {
        return (float)Math.sqrt(Math.pow(outlets.get(outlet1).getX() - outlets.get(outlet2).getX() , 2) + Math.pow(outlets.get(outlet1).getY() - outlets.get(outlet2).getY(), 2));
    }

    public float calculateDeliveryCost() {
        List<Integer> listOfOutlets = getListOfOutlets();
        float totalDistance = 0;
        float totalCost;
        for (int current = 0; current < listOfOutlets.size() - 1; current++) {
            totalDistance += getDistanceBetweenTwoOutlets(listOfOutlets.get(current), listOfOutlets.get(current + 1));
        }
        totalCost = totalDistance * fuelCostPerUnit;
        return totalCost;
    }
}

class Simulation {
    protected Settlement simulationSettlement;
    protected int noOfCompanies;
    protected float fuelCostPerUnit, baseCostForDelivery;
    protected List<Company> companies = new ArrayList();
    private static Random rnd = new Random();

    public Simulation() {
        fuelCostPerUnit = 0.0098f;
        baseCostForDelivery = 100;
        String choice;
        Console.write("Enter L for a large settlement, anything else for a normal size settlement: ");
        choice = Console.readLine();
        if (choice.equals("L")) {
            int extrax, extray, extraHouseholds;
            Console.write("Enter additional amount to add to x size of settlement: ");
            extrax = Integer.parseInt(Console.readLine());
            Console.write("Enter additional amount to add to y size of settlement: ");
            extray = Integer.parseInt(Console.readLine());
            Console.write("Enter additional number of households to add to settlement: ");
            extraHouseholds = Integer.parseInt(Console.readLine());
            simulationSettlement = new LargeSettlement(extrax, extray, extraHouseholds);
        } else {
            simulationSettlement = new Settlement();            
        }
        Console.write("Enter D for default companies, anything else to add your own start companies: ");
        choice = Console.readLine();
        if (choice.equals("D")) {
            noOfCompanies = 3;
            Company company1 = new Company("AQA Burgers", "fast food", 100000, 200, 203, fuelCostPerUnit, baseCostForDelivery);
            companies.add(company1);
            companies.get(0).openOutlet(300, 987);
            companies.get(0).openOutlet(500, 500);
            companies.get(0).openOutlet(305, 303);
            companies.get(0).openOutlet(874, 456);
            companies.get(0).openOutlet(23, 408);
            companies.get(0).openOutlet(412, 318);
            Company company2 = new Company("Ben Thor Cuisine", "named chef", 100400, 390, 800, fuelCostPerUnit, baseCostForDelivery);
            companies.add(company2);
            Company company3 = new Company("Paltry Poultry", "fast food", 25000, 800, 390, fuelCostPerUnit, baseCostForDelivery);
            companies.add(company3);
            companies.get(2).openOutlet(400, 390);
            companies.get(2).openOutlet(820, 370);
            companies.get(2).openOutlet(800, 600);
        } else {
            Console.write("Enter number of companies that exist at start of simulation: ");
            noOfCompanies = Integer.parseInt(Console.readLine());
            for (int count = 1; count <= noOfCompanies; count++) {
                addCompany();
            }
        }
    }

    public void displayMenu() {
        Console.writeLine(System.lineSeparator() + "*********************************");
        Console.writeLine("**********    MENU     **********");
        Console.writeLine("*********************************");
        Console.writeLine("1. Display details of households");
        Console.writeLine("2. Display details of companies");
        Console.writeLine("3. Modify company");
        Console.writeLine("4. Add new company");
        Console.writeLine("6. Advance to next day");
        Console.writeLine("Q. Quit");
        Console.write(System.lineSeparator() + "Enter your choice: ");
    }

    private void displayCompaniesAtDayEnd() {
        String details;
        Console.writeLine(System.lineSeparator() + "**********************");
        Console.writeLine("***** Companies: *****");
        Console.writeLine("**********************" + System.lineSeparator());
        for (Company c : companies) {
            Console.writeLine(c.getName());
            Console.writeLine();
            details = c.processDayEnd();
            Console.writeLine(details + System.lineSeparator());
        }
    }

    private void processAddHouseholdsEvent() {
        int noOfNewHouseholds = (int)(rnd.nextFloat() * 4) + 1;
        for (int count = 1; count <= noOfNewHouseholds; count++) {
            simulationSettlement.addHousehold();
        }
        Console.writeLine(noOfNewHouseholds + " new households have been added to the settlement.");
    }

    private void processCostOfFuelChangeEvent() {
        float fuelCostChange = ((int)(rnd.nextFloat() * 9) + 1) / 10f;
        int upOrDown = (int)(rnd.nextFloat() * 2);
        int companyNo = (int)(rnd.nextFloat() * companies.size());
        if (upOrDown == 0) {
            Console.writeLine("The cost of fuel has gone up by " + fuelCostChange + " for " + companies.get(companyNo).getName());
        } else {
            Console.writeLine("The cost of fuel has gone down by " + fuelCostChange + " for " + companies.get(companyNo).getName());
            fuelCostChange *= -1;
        }
        companies.get(companyNo).alterFuelCostPerUnit(fuelCostChange);
    }

    private void processReputationChangeEvent() {
        float reputationChange = ((int)(rnd.nextFloat() * 9) + 1) / 10f;
        int upOrDown = (int)(rnd.nextFloat() * 2);
        int companyNo = (int)(rnd.nextFloat() * companies.size());
        if (upOrDown == 0) {
            Console.writeLine("The reputation of " + companies.get(companyNo).getName() + " has gone up by " + reputationChange);
        } else {
            Console.writeLine("The reputation of " + companies.get(companyNo).getName() + " has gone down by " + reputationChange);
            reputationChange *= -1;
        }
        companies.get(companyNo).alterReputation(reputationChange);
    }

    private void processCostChangeEvent() {
        int costToChange = (int)(rnd.nextFloat() * 2);
        int upOrDown = (int)(rnd.nextFloat() * 2);
        int companyNo = (int)(rnd.nextFloat() * companies.size());
        float amountOfChange;
        if (costToChange == 0) {
            amountOfChange = ((int)(rnd.nextFloat() * 19) + 1) / 10f;
            if (upOrDown == 0) {
                Console.writeLine("The daily costs for " + companies.get(companyNo).getName() + " have gone up by " + amountOfChange);
            } else {
                Console.writeLine("The daily costs for " + companies.get(companyNo).getName() + " have gone down by " + amountOfChange);
                amountOfChange *= -1;
            }
            companies.get(companyNo).alterDailyCosts(amountOfChange);
        } else {
            amountOfChange = (int)((rnd.nextFloat() * 9) + 1) / 10f;
            if (upOrDown == 0) {
                Console.writeLine("The average cost of a meal for " + companies.get(companyNo).getName() + " has gone up by " + amountOfChange);
            } else {
                Console.writeLine("The average cost of a meal for " + companies.get(companyNo).getName() + " has gone down by " + amountOfChange);
                amountOfChange *= -1;
            }
            companies.get(companyNo).alterAvgCostPerMeal(amountOfChange);
        }
    }

    private void displayEventsAtDayEnd() {
        Console.writeLine(System.lineSeparator() + "***********************");
        Console.writeLine("*****   Events:   *****");
        Console.writeLine("***********************" + System.lineSeparator());
        float eventRanNo;
        eventRanNo = rnd.nextFloat();
        if (eventRanNo < 0.25f) {
            eventRanNo = rnd.nextFloat();
            if (eventRanNo < 0.25f) {
                processAddHouseholdsEvent();
            }
            eventRanNo = rnd.nextFloat();
            if (eventRanNo < 0.5f) {
                processCostOfFuelChangeEvent();
            }
            eventRanNo = rnd.nextFloat();
            if (eventRanNo < 0.5f) {
                processReputationChangeEvent();
            }
            eventRanNo = rnd.nextFloat();
            if (eventRanNo >= 0.5f) {
                processCostChangeEvent();
            }
        } else {
            Console.writeLine("No events.");
        }
    }

    public void processDayEnd() {
        float totalReputation = 0;
        List<Float> reputations = new ArrayList();
        int companyRNo, current, loopMax, x, y;
        for (Company c : companies) {
            c.newDay();
            totalReputation += c.getReputationScore();
            reputations.add(totalReputation);
        }
        loopMax = simulationSettlement.getNumberOfHouseholds() - 1;
        for (int counter = 0; counter <= loopMax; counter++) {
            int[] tempLocation = simulationSettlement.getHouseholdLocation(counter);
            x = tempLocation[0];
            y = tempLocation[1];                
            if (simulationSettlement.findOutIfHouseholdEatsOut(counter, x, y)) {
                companyRNo = (int)(rnd.nextFloat() * ((int)(totalReputation) + 1));
                current = 0;
                while (current < reputations.size()) {                    
                    if (companyRNo < reputations.get(current)) {
                        companies.get(current).addVisitToNearestOutlet(x, y);
                        break;
                    }
                    current += 1;
                }
            }
        }
        displayCompaniesAtDayEnd();
        displayEventsAtDayEnd();
    }

    public void addCompany() {
        int balance, x, y;
        String companyName, typeOfCompany;
        Console.write("Enter a name for the company: ");
        companyName = Console.readLine();
        Console.write("Enter the starting balance for the company: ");
        balance = Integer.parseInt(Console.readLine());
        do {
            Console.write("Enter 1 for a fast food company, 2 for a family company or 3 for a named chef company: ");
            typeOfCompany = Console.readLine();
        } while (!typeOfCompany.equals("1") && !typeOfCompany.equals("2") && !typeOfCompany.equals("3"));
        if (typeOfCompany.equals("1")) {
            typeOfCompany = "fast food";
        } else if (typeOfCompany.equals("2")) {
            typeOfCompany = "family";
        } else {
            typeOfCompany = "named chef";
        }
        int[] tempLocation = simulationSettlement.getRandomLocation();
        x = tempLocation[0];
        y = tempLocation[1];
        Company newCompany = new Company(companyName, typeOfCompany, balance, x, y, fuelCostPerUnit, baseCostForDelivery);
        companies.add(newCompany);
    }

    public int getIndexOfCompany(String companyName) {
        int index = -1;
        for (int current = 0; current < companies.size(); current++) {
            if (companies.get(current).getName().toLowerCase().equals(companyName.toLowerCase())) {
                return current;
            }
        }
        return index;
    }

    public void modifyCompany(int index) {
        String choice;
        int outletIndex, x, y;
        boolean closeCompany;
        Console.writeLine(System.lineSeparator() + "*********************************");
        Console.writeLine("*******  MODIFY COMPANY   *******");
        Console.writeLine("*********************************");
        Console.writeLine("1. Open new outlet");
        Console.writeLine("2. Close outlet");
        Console.writeLine("3. Expand outlet");
        Console.write(System.lineSeparator() + "Enter your choice: ");
        choice = Console.readLine();
        Console.writeLine();
        if (choice.equals("2") || choice.equals("3")) {
            Console.write("Enter id of outlet: ");
            outletIndex = Integer.parseInt(Console.readLine());
            if (outletIndex > 0 && outletIndex <= companies.get(index).getNumberOfOutlets()) {
                if (choice.equals("2")) {
                    closeCompany = companies.get(index).closeOutlet(outletIndex - 1);
                    if (closeCompany) {
                        Console.writeLine("That company has now closed down as it has no outlets.");
                        companies.remove(index);
                    }
                } else {
                    companies.get(index).expandOutlet(outletIndex - 1);
                }
            } else {
                Console.writeLine("Invalid outlet id.");
            }
        } else if (choice.equals("1")) {
            Console.write("Enter X coordinate for new outlet: ");
            x = Integer.parseInt(Console.readLine());
            Console.write("Enter Y coordinate for new outlet: ");
            y = Integer.parseInt(Console.readLine());
            if (x >= 0 && x < simulationSettlement.getXSize() && y >= 0 && y < simulationSettlement.getYSize()) {
                companies.get(index).openOutlet(x, y);
            } else {
                Console.writeLine("Invalid coordinates.");
            }
        }
        Console.writeLine();
    }

    public void displayCompanies() {
        Console.writeLine(System.lineSeparator() + "*********************************");
        Console.writeLine("*** Details of all companies: ***");
        Console.writeLine("*********************************" + System.lineSeparator());
        for (Company c : companies) {
            Console.writeLine(c.getDetails() + System.lineSeparator());
        }
        Console.writeLine();
    }

    public void run() {
        String choice = "";
        int index;
        while (!choice.equals("Q")) {
            displayMenu();
            choice = Console.readLine();
            switch (choice) {
                case "1":
                    simulationSettlement.displayHouseholds();
                    break;
                case "2":
                    displayCompanies();
                    break;
                case "3":
                    String companyName;
                    do {
                        Console.write("Enter company name: ");
                        companyName = Console.readLine();
                        index = getIndexOfCompany(companyName);
                    } while (index == -1);
                    modifyCompany(index);
                    break;
                case "4":
                    addCompany();
                    break;
                case "6":
                    processDayEnd();
                    break;
                case "Q":
                    Console.writeLine("Simulation finished, press Enter to close.");
                    Console.readLine();
                    break;
            }
        }
    }
}

public class Paper1_ALvl_2020_Java_Pre_ {
    
    public static void main(String[] args) {
        Simulation thisSim = new Simulation();
        thisSim.run();
    }
}